/***********************************************************************
 * Sim6811.c                                                           *
 *                                                                     *
 * This program is invoked by the user from the command line.  It then *
 * examines the user's environment to determine which version of the   *
 * Motorola 6811 Simulator should be run.                              *
 *                                                                     *
 * If the user's terminal type is a character terminal, Sim6811_CUI is *
 * invoked.  Otherwise, Sim6811_GUI is invoked.                        *
 ***********************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void main(int argc, char* argv[], char* envp[])
{
    char display[255] = "none", command[255], temp[255];
    char installPath[] = "/usr/unsupported/src/Sim6811/";

    int  i, search = 1;

    fprintf(stderr, "\n  Analyzing Environment ... ");

    /* Search the user's environment for the existence of the
       DISPLAY variable.  */ 
    while( *envp && search ) {
	if( !strncmp(envp[0], "DISPLAY", 7) ) {
	    /* Store the value of DISPLAY in display */
	    sprintf(display, "%s", envp[0]+8);

	    /* DISPLAY has been found. */
	    search = 0;
	}
	
	/* Move to next environment variable. */
	envp++;
    }

    /* If DISPLAY was not found, assume the user is running on 
       a character-based terminal.  Then build the command to
       execute the appropriate version of the simulator. */
    if ( !strcmp(display, "none") ) {
	/* Complete the "Analyzing Environment ..." info line */
	fprintf(stderr, "No Graphics Capability\n");

	/* Notify user which simulator is being started. */
	fprintf(stderr, "  Starting Text-based Simulator ...\n\n");
	fprintf(stderr, 
		"  Note: The Text-based version requires one argument:\n");

/*	sprintf(command, "/usr/unsupported/src/Sim6811/sim6811_%s", "CUI"); */
	sprintf(command, "%ssim6811_%s", installPath, "CUI");
/*	sprintf(command, "6811sim/NewGUI/source/sim6811_%s", "CUI"); */
    }
    else {
	/* Complete the "Analyzing Environment ..." info line */
	fprintf(stderr, "Detected Graphics Capability\n");

	/* Notify user which simulator is being started. */
	fprintf(stderr, 
		"  Starting GUI Simulator ... (in background mode)\n\n");
	fprintf(stderr, "Please wait ...\n");

/*	sprintf(command, "/usr/unsupported/src/Sim6811/sim6811_%s &", "GUI");*/
	sprintf(command, "%ssim6811_%s &", installPath, "GUI");
/*	sprintf(command, "6811sim/NewGUI/sim6811_%s &", "GUI");  */
    }

    /* Pass along the command line arguments specified by the user */
    for (i=1; i<argc; i++) {
	sprintf(temp, " %s", argv[i]);
	strcat(command, temp);
    }

    /* Perform the system call to execute the selected simulator. */
    system( command );
}
