/******************************************************************
 * convert.c -- This file contains all the routines used for the  *
 *              Base Conversion Utility (for CS273's Sim6811_GUI) *
 *                                                                *
 *                   written by Rob Sinclair                      *
 *                    rsinclai@cs.nmsu.edu                        *
 ******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "sim6811_ui.h"
#include "defs.h"


extern  sim6811_winBaseConversion_objects   *Sim6811_winBaseConversion;


/****************************************************************
 * This function effectively inverts all the bits in the bit
 * pattern 'bits'.
 ****************************************************************/
char *invertBits(char *bits)
{
    int   i, len;
    char  inverted[40]="\0";

    len = strlen(bits);

    for (i=0; i<len; i++) {
	strcat(inverted, ((bits[i]=='1') ? "0" : "1"));
    }

    return inverted;
}


/****************************************************************
 * This function converts a binary representation of a number 
 * into its corresponding integer value.
 ****************************************************************/
int BinToDec(char* binValue, int signedValue)
{
    int   i, len, neg=0, accum=0, add=1;

    len = strlen(binValue);

    if (len > 0) {
	/* If the binary value is a signed, negative number, perform
	   two's-complement conversion. */
	if (signedValue && (strncmp(binValue, "1", 1)==0)) {
	    neg = TRUE;
	    strcpy(binValue, invertBits(binValue));
	}

	/* Calculate the decimal value of the bit string. */
	for (i=len-1; i>=0; --i) {
	    if (binValue[i]=='1')
	      accum += add;

	    add *= 2;
	}

	/* Finish the two's-complement conversion, if the binary
	   value is negative. */
	if (neg) {
	    ++accum;
	    accum *= -1;
	}
    }

    return accum;
}


/****************************************************************
 * This function converts a decimal value into its binary 
 * representation.  The binary representation is returned as a
 * string of 1's and 0's.
 ****************************************************************/
char *DecToBin(int value)
{
    int            i, mask = 1<<31;
    char           binValue[40]="\0";

    for (i=1; i<=32; ++i) {
	strcat(binValue, ((value & mask) ? "1" : "0"));
	value <<= 1;

	if ((i%8 == 0) && (i != 32))
	  strcat(binValue, " ");
    }

    return binValue;
}


/****************************************************************
 * This function gets the initial value and its base, and converts
 * it to the requested base.  It then displays the converted
 * value in the textFinalValue field.
 ****************************************************************/
void convertValue()
{
    int  initBase, newBase, tempValue, len;
    char initValue[10], newValue[10], tempChar;

    strcpy(initValue,(char*)xv_get(Sim6811_winBaseConversion->textInitalValue,
				   PANEL_VALUE, NULL));

    if (strlen(initValue) < 1) return;

    initBase = (int)xv_get(Sim6811_winBaseConversion->setInitalBase, 
			   PANEL_VALUE, NULL);
    newBase  = (int)xv_get(Sim6811_winBaseConversion->setNewBase,
			   PANEL_VALUE, NULL);

    if (initBase == newBase) {
	xv_set(Sim6811_winBaseConversion->textFinalValue,
	       PANEL_VALUE, initValue, NULL);
    }
    else {
	/* Convert from the initial value/character to an integer. */
	switch( initBase ) {
	  case ASCII:
	    if ( strlen(initValue)>1 ) {
		xv_set(Sim6811_winBaseConversion->textFinalValue, PANEL_VALUE,
		       "Only one character can be converted.", NULL);
		return;
	    }
	    sscanf(initValue, "%c", &tempChar);
	    tempValue = (int)tempChar;	    
	    break;
	  case DECIMAL:
	    sscanf(initValue, "%d", &tempValue);
	    break;
	  case HEX:
	    sscanf(initValue, "%x", &tempValue);
	    break;
	  case OCTAL:
	    sscanf(initValue, "%o", &tempValue);
	    break;
	  case SIGNED_BINARY:
	    tempValue = BinToDec(initValue, TRUE);

	    /* Redisplay the user-specified value in its 32-bit rep. */
/*	    xv_set(Sim6811_winBaseConversion->textInitalValue,
		   PANEL_VALUE, DecToBin(tempValue), NULL);
*/	    break;
	  case UNSIGNED_BINARY:
	    tempValue = BinToDec(initValue, FALSE);

	    /* Redisplay the user-specified value in its 32-bit rep. */
/*	    xv_set(Sim6811_winBaseConversion->textInitalValue,
		   PANEL_VALUE, DecToBin(tempValue), NULL);
*/	    break;
	}

	/* Convert from the integer value to the final value/character. */
	switch( newBase ) {
	  case ASCII:
	    if ( tempValue>0 && isprint(tempValue) )
	      sprintf(newValue, "%c", tempValue);
	    else
	      sprintf(newValue, "non-printable character");
	    break;
	  case DECIMAL:
	    sprintf(newValue, "%d", tempValue);
	    break;
	  case HEX:
	    sprintf(newValue, "%x", tempValue);
	    break;
	  case OCTAL:
	    sprintf(newValue, "%o", tempValue);
	    break;
	  case SIGNED_BINARY:
	    strcpy(newValue, DecToBin(tempValue));
	    break;
	}

	/* Display the new value */
	xv_set(Sim6811_winBaseConversion->textFinalValue,
	       PANEL_VALUE, newValue, NULL);
    }
}


