/* simulator for 6811: inline definitions

$Log: inline.h,v $
 * Revision 1.1  1993/05/29  18:56:31  ted
 * Initial revision
 *

*/

#include "../defs.h"

#ifndef INLINE
#define INLINE
#endif

/* setter functions for status bits */
INLINE void set_half_carry(m6811 *state, int value)
{state->half_carry = value;}

INLINE void set_x_interrupt(m6811 *state, int value)
{state->x_interrupt = value;}

INLINE void set_stop_disable(m6811 *state, int value)
{state->stop_disable = value;}

INLINE void set_overflow(m6811 *state, int value)
{state->overflow = value;}

INLINE void set_negative(m6811 *state, int value)
{state->negative = value;}

INLINE void set_carry(m6811 *state, int value)
{state->carry = value;}

INLINE void set_idisable(m6811 *state, int value)
{state->idisable = value;}

INLINE void set_zero(m6811 *state, int value)
{state->zero = value;}

/* getter functions for conditions code bits */
INLINE int get_half_carry(m6811 *state)
{return state->half_carry;}

INLINE int get_x_interrupt(m6811 *state)
{return state->x_interrupt;}

INLINE int get_stop_disable(m6811 *state)
{return state->stop_disable;}

INLINE int get_overflow(m6811 *state)
{return state->overflow;}

INLINE int get_negative(m6811 *state)
{return state->negative;}

INLINE int get_carry(m6811 *state)
{return state->carry;}

INLINE int get_idisable(m6811 *state)
{return state->idisable;}

INLINE int get_zero(m6811 *state)
{return state->zero;}

/* getter functions for registers */
INLINE byte get_a(m6811 *state)
{return (state->d >> 8) & 0xff;}

INLINE byte get_b(m6811 *state)
{return state->d & 0xff;}

INLINE word get_d(m6811 *state)
{return state->d & 0xffffL;}

INLINE word get_x(m6811 *state)
{return state->x & 0xffffL;}

INLINE word get_y(m6811 *state)
{return state->y & 0xffffL;}

INLINE word get_s(m6811 *state)
{return state->sp & 0xffffL;}

INLINE word get_pc(m6811 *state)
{return state->pc & 0xffffL;}

/* setter functions for registers */
INLINE void set_a(m6811 *state, byte value)
{
    state->d &= 0xff;
    state->d |= value<<8;
}

INLINE void set_b(m6811 *state, byte value)
{
    state->d &= 0xff00;
    state->d |= value;
}

INLINE void set_d(m6811 *state, word value)
{state->d = 0xffff&value;}

INLINE void set_x(m6811 *state, word value)
{state->x = 0xffff&value;}

INLINE void set_y(m6811 *state, word value)
{state->y = 0xffff&value;}

INLINE void set_s(m6811 *state, word value)
{state->sp = 0xffff&value;}

INLINE void set_pc(m6811 *state, word value)
{state->pc = 0xffff&value;}

/* memory modifiers */
INLINE void set_mem8(byte *m, word add, byte value)
{m[INDEX(add)] = 0xff&value;}

INLINE void set_mem16(byte *m, word add, word value)
{add = INDEX(add); m[add]=(value>>8)&0xff; m[add+1] = value&0xff;}

/* memory accessors */
INLINE byte get_mem8(byte *m, word add)
{add = INDEX(add); return 0xff&m[add];}

INLINE word get_mem16(byte *m, word add)
{add = INDEX(add);return (m[add]<<8) + m[add+1];}

/* read memory, bump pc.  as in reading op-codes */
INLINE byte fetch8(byte *m, m6811 *state)
{return get_mem8(m, state->pc++);}

INLINE word fetch16(byte *m, m6811 *state)
{long r;r = get_mem16(m, state->pc);state->pc += 2; return r;}

/* 8 bit add, sets condition codes */
INLINE byte add8(byte a, byte b, m6811 *state)
{
    int r;

    r = a+b;
    set_carry(state, (0x100&r) != 0);
    set_zero(state, (r&0xff) == 0);
    set_negative(state, (r&0x80) != 0);
    set_overflow(state,
		 (((a^b)&0x80) == 0) &&
		 ((((a&0x80)!=0)^get_negative(state))!=0));
    return r&0xff;
}

INLINE byte sub8(byte a, byte b, m6811 *state)
{return add8(a, -b, state);}
                                
/* 8 bit OR, just like add, but hits fewer condition codes */
INLINE byte or8(byte a, byte b, m6811 *state)
{
    int r;

    r = a|b;
    set_zero(state, (r&0xff) == 0);
    set_negative(state, (r&0x80) != 0);
    set_overflow(state, 0);
    return r&0xff;
}

/* AND works just like OR */ 
INLINE byte and8(byte a, byte b, m6811 *state)
{
    int r;

    r = a&b;
    set_zero(state, (r&0xff) == 0);
    set_negative(state, (r&0x80) != 0);
    set_overflow(state, 0);
    return r&0xff;
}

/* and so does XOR */
INLINE byte xor8(byte a, byte b, m6811 *state)
{
    int r;

    r = a^b;
    set_zero(state, (r&0xff) == 0);
    set_negative(state, (r&0x80) != 0);
    set_overflow(state, 0);
    return r&0xff;
}

/* used in loads and stores */
INLINE void tst8(byte a, m6811 *state)
{
    set_zero(state, (a&0xff) == 0);
    set_negative(state, (a&0x80) != 0);
    set_overflow(state, 0);
}

/* used when loading and storing D */
INLINE void tst16(word a, m6811 *state)
{
    set_zero(state, (a&0xffff) == 0);
    set_negative(state, (a&0x8000) != 0);
    set_overflow(state, 0);
}

/* increment a 16 bit quantity, set flags */
INLINE word inc16(word old, word inc, m6811 *state)
{
    old += inc;
    set_zero(state, (old&0xffff) == 0);
    return old&0xffff;
}

/* same for 8 bits.  more conditions are checked */
INLINE byte inc8(byte old, byte inc, m6811 *state)
{
    int t;
    t = old;
    t += inc;
    set_zero(state, (t&0xff) == 0);
    set_negative(state, (t & 0x80) != 0);
    set_overflow(state, t > 256);
    return t&0xff;
}

/* shift from a specified bit, into carry.  general enough to
   handle all 6811 shifts */
INLINE byte shift8(byte old, int direction, int new_bit, m6811 *state)
{
    int carry, t;
    t = old;
    if (direction == LEFT) {
	carry = ((t & 0x80) != 0);
	t = (t<<1) + new_bit;
    }
    else if (direction == RIGHT) {
	carry = t & 1;
	t = (t>>1) + (new_bit<<7);
    }
    else {
	output(STDERR, "internal error in shift8\n");
	exit(1);
    }
    set_carry(state, carry);
    tst8(t&0xff, state);
    return t&0xff;
}

INLINE word shift16(word old, int direction, int new_bit, m6811 *state)
{
    int carry;
    if (direction == LEFT) {
	carry = ((old & 0x8000) != 0);
	old = (old<<1) + new_bit;
    }
    else if (direction == RIGHT) {
	carry = old & 1;
	old = (old>>1) + (new_bit<<7);
    }
    else {
	output(STDERR, "internal error in shift16\n");
	exit(1);
    }
    set_carry(state, carry);
    tst16(old&0xffff, state);
    return old&0xffff;
}

/* 16 bit add with condition codes */
INLINE word add16(word a, word b, m6811 *state)
{
    long r;

    r = a+b;
    set_carry(state, r>65536L);
    set_zero(state, (r&0xffff) == 0);
    set_negative(state, (r&0x8000) != 0);
    set_overflow(state, get_carry(state) ^ get_negative(state));
    return r&0xff;
}

/* and subtract */
INLINE word sub16(word a, word b, m6811 *state)
{
    return add16(a, -b, state);
}
    
/* count some cycles */
INLINE void cycle(m6811 *state, int count)
{state->t += count;}

