/*******************************************************************
 * output.c                                                        *
 *                                                                 *
 * This file contains the output() function.  This function takes  *
 * a variable number of arguments and is structured just like the  *
 * fprintf() function.  The only difference is that the first      *
 * argument must be one of the window constants defined in defs.h. *
 *                                                                 *
 * This function allows the Motorola 6811 Simulator's Makefile to  *
 * create both versions (GUI and text-based) from the same source  *
 * code files (dependent upon the definition of the symbol GUI).   *
 * Since the program's output is the only difference between these *
 * two versions, this function uses the GUI symbol to decide if it *
 * should send ALL output to stderr, or if it should send it to    *
 * the requested window.                                           *
 *                                                                 *
 *******************************************************************/

/*******************************************************************
 * Examples:                                                       *
 *                                                                 *
 *   output(STDERR, "Using %d arguments: %s %s %s %s %s.\n",       *
 *	      5, "FIRST", "SECOND", "THIRD", "FOURTH", "FIFTH");   *
 *   output(STDOUT, "\n");                                         *
 *   output(WIN_MEMORY, "Address 0x0123 = 0x%d\n", 43);            *
 *   output(WIN_OUTPUT, "Assembly #%d %s.", 32, "succeeded");      *
 *                                                                 *
 *******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include "../defs.h"

#ifdef GUI
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include "../sim6811_ui.h"

extern  sim6811_sim6811Window_objects	*Sim6811_sim6811Window;
extern  sim6811_winSourceCode_objects	*Sim6811_winSourceCode;
extern  sim6811_winCrossRef_objects	*Sim6811_winCrossRef;
extern  sim6811_winCircuitBoard_objects	*Sim6811_winCircuitBoard;
extern  sim6811_winMemory_objects	*Sim6811_winMemory;
extern  sim6811_winRegisters_objects	*Sim6811_winRegisters;
extern  sim6811_winStack_objects	*Sim6811_winStack;
extern  sim6811_winSymbolTable_objects	*Sim6811_winSymbolTable;
extern  sim6811_winOutput_objects	*Sim6811_winOutput;
extern  sim6811_winEditMemory_objects	*Sim6811_winEditMemory;
extern  sim6811_winMultiStep_objects	*Sim6811_winMultiStep;
extern  sim6811_winBreakpoints_objects	*Sim6811_winBreakpoints;
#endif

void popStack()
{
#ifdef GUI
    xv_set(Sim6811_winStack->listStack, PANEL_LIST_DELETE, 0, NULL);
#endif
}

void pushStack(int value) 
{
#ifdef GUI
    char temp[8];
    sprintf(temp, "0x%02x", value);
    xv_set(Sim6811_winStack->listStack, 
	   PANEL_LIST_INSERT, 0,
	   PANEL_LIST_STRING, 0, temp, NULL);
#endif
}

void output(int Dest, ...)
{
    int      i=0, num=0, t=0, addr=0;
    va_list  arg;
    char     fmtstr[255], buf[500], C, *text, temp[30],
             A[5], B[5], D[10], X[5], Y[5], SP[10], PC[10];

    text = buf;

    /* Initialize the argument pointer, arg. */
    va_start(arg, Dest);

    /* Get the format string from the stack. */
    strcpy(fmtstr, va_arg(arg, char*));

    /* Perform the appropriate type of output */
    switch( Dest ) {

#ifdef GUI
      case WIN_BREAKPTS:
	/* Build the string to be displayed */
	vsprintf(text, fmtstr, arg);

	/* If "      breakpoints -> " is being printed, ignore it. */
       	if ((num = sscanf(text, "	breakpoints %s ", A)) == 1) {
	    /* Swallow the string. */
      	    break;
	}
	else {
	    /* -1 -> Permanent Breakpoint
	       +n -> Temporary Breakpoint that will exist for n occurances */

	    /* If pattern matches "%04x " then breakpt is permanent */
/*	    if ((num = sscanf(text, "%x", &addr)) == 1) {
		    sprintf(text, "0x%04x (Permanent)", t);
		    xv_set(Sim6811_winBreakpoints->listBreakPts, 
			   PANEL_LIST_INSERT, 0,
			   PANEL_LIST_STRING, 0, text,
			   NULL);
	    }
	    else {
*/		/* If pattern matches " %04x[%d] " then breakpt is temp */
/*		if ((num = sscanf(text, "%x[%d]", &addr, &t)) == 2) {
		    sprintf(text, "0x%04x (Temporary ... %d)", addr, t);
		    xv_set(Sim6811_winBreakpoints->listBreakPts, 
			   PANEL_LIST_INSERT, 0,
			   PANEL_LIST_STRING, 0, text,
			   NULL);
		}
	    }
*/	}
	break;

      case WIN_OUTPUT:
	/* Build the string to be displayed */
	vsprintf(text, fmtstr, arg);

	/* Insert the string in the window */
	textsw_insert(Sim6811_winOutput->textOutput, text, strlen(text));
	break;

      case WIN_MEMORY:
	/* Build the string to be displayed */
	vsprintf(text, fmtstr, arg);

	/* Insert the string in the correct line of the memory window */
	sscanf(text, "%x", &addr);
DEBUG(fprintf(stderr, "Changing Mem Addr 0x%04x.\n%s\n", addr, text));
        text += 2;
DEBUG(fprintf(stderr, "text (after addition: %s\n", text));
	switch( addr ) {
	  case 0:
	    xv_set(Sim6811_winMemory->txtMem0, PANEL_VALUE, text, NULL);
	    break;
	  case 1:
	    xv_set(Sim6811_winMemory->txtMem1, PANEL_VALUE, text, NULL);
	    break;
	  case 2:
	    xv_set(Sim6811_winMemory->txtMem2, PANEL_VALUE, text, NULL);
	    break;
	  case 3:
	    xv_set(Sim6811_winMemory->txtMem3, PANEL_VALUE, text, NULL);
	    break;
	  case 4:
	    xv_set(Sim6811_winMemory->txtMem4, PANEL_VALUE, text, NULL);
	    break;
	  case 5:
	    xv_set(Sim6811_winMemory->txtMem5, PANEL_VALUE, text, NULL);
	    break;
	  case 6:
	    xv_set(Sim6811_winMemory->txtMem6, PANEL_VALUE, text, NULL);
	    break;
	  case 7:
	    xv_set(Sim6811_winMemory->txtMem7, PANEL_VALUE, text, NULL);
	    break;
	  case 8:
	    xv_set(Sim6811_winMemory->txtMem8, PANEL_VALUE, text, NULL);
	    break;
	  case 9:
	    xv_set(Sim6811_winMemory->txtMem9, PANEL_VALUE, text, NULL);
	    break;
	  case 10:
	    xv_set(Sim6811_winMemory->txtMem10, PANEL_VALUE, text, NULL);
	    break;
	  case 11:
	    xv_set(Sim6811_winMemory->txtMem11, PANEL_VALUE, text, NULL);
	    break;
	  case 12:
	    xv_set(Sim6811_winMemory->txtMem12, PANEL_VALUE, text, NULL);
	    break;
	  case 13:
	    xv_set(Sim6811_winMemory->txtMem13, PANEL_VALUE, text, NULL);
	    break;
	  case 14:
	    xv_set(Sim6811_winMemory->txtMem14, PANEL_VALUE, text, NULL);
	    break;
	  case 15:
	    xv_set(Sim6811_winMemory->txtMem15, PANEL_VALUE, text, NULL);
	    break;
	}

	break;

      case WIN_REGISTERS:
	/* Build the string to be displayed */
	vsprintf(text, fmtstr, arg);

	/* See if registers are being printed */
	num = sscanf(text, "\nA %s B %s    t=%d\nD  %s\nX  %s Y  %s ", 
		     A, B, &t, D, X, Y);

DEBUG(fprintf(stderr, "Updating Register Window with:\n\t%s\n\n", text));

	/* Update registers if the scan matched previous pattern */
	if (num == 6) {
	    xv_set(Sim6811_winRegisters->txtRegA, PANEL_VALUE, A, NULL);
	    xv_set(Sim6811_winRegisters->txtRegB, PANEL_VALUE, B, NULL);
	    xv_set(Sim6811_winRegisters->txtRegD, PANEL_VALUE, D, NULL);
	    xv_set(Sim6811_winRegisters->txtRegX, PANEL_VALUE, X, NULL);
	    xv_set(Sim6811_winRegisters->txtRegY, PANEL_VALUE, Y, NULL);

	    sprintf(temp, "Cycles: %4d", t);
	    xv_set(Sim6811_sim6811Window->sim6811Window, 
		   FRAME_RIGHT_FOOTER, temp, NULL);

	}
	else {
	    /* See if SP and PC are being printed */
	    num = sscanf(text, "SP %s PC %s ", SP, PC);
DEBUG(fprintf(stderr, "Printing to REG_WIN: SP=%s PC=%s.\n", SP, PC));

	    /* Update SP and PC if the scan matched previous pattern */
	    if (num == 2){
		xv_set(Sim6811_winRegisters->txtRegSP, PANEL_VALUE, SP, NULL);
		xv_set(Sim6811_winRegisters->txtRegPC, PANEL_VALUE, PC, NULL);
	    }
	    else {
		/* See if Condition Codes are being printed */
		num = sscanf(text, "%c%c (%c%c%c%c %c%c%c%c)",
			     &C,&C,&C,&C,&C,&C,&C,&C,&C,&C);

		/* Update Condition Codes */
		if (num == 10){
		    xv_set(Sim6811_winRegisters->txtRegCCodes, 
			   PANEL_VALUE, text, NULL);
		}
	    }
	}
	break;
#endif

      case STDOUT:
	vprintf(fmtstr, arg);
	break;

      default:  /* Send output to stderr by default */
	vfprintf(stderr, fmtstr, arg);
	break;
    }

    /* Clean up */
    va_end(arg);
}

