typedef long value;		/* really any long-sized datum */

typedef struct s_node *node;
typedef struct s_node {
    void *key;			/* again, we don't really know what type */
    value v;
    node forward[1];		/* variable sized array of skip pointers */
} _node_el;

/* note that all size routines must report a large enough size to
   allow for any alignment constraints... rounding sizeof up may work,
   but the best is to take the diff between addresses of consecutive
   array elements */
typedef struct {
    int level;			/* the number of levels in list */
    int (*compare)();		/* comparison for keys */
    int (*key_copy)();		/* copier for keys */
    int (*key_size)();		/* returns size of key */
    int (*key_free)();		/* deallocates key */
    int (*size)();		/* returns size of data value */
    int (*copy)();		/* copier for data value */
    int (*free)();		/* deallocates data */
    struct s_node *header;	/* and the actual skip list */
} *skip_list;

/* create any kind of list */
skip_list create_list_with_ops(intfun k_cmp,
			       intfun k_copy, intfun k_size, intfun k_free,
			       intfun v_copy, intfun v_size, intfun v_free);

/* create a list of char*'s to values */
skip_list create_skip_list();

/* deallocates a skip list as thoroughly as we know how */
void free_skip_list( skip_list l );

/* insert a key/value into a skip list.  if replace_dups is set,
   then any previous entry with the same key will be overwritten.
   otherwise, new entries precede any previous entries */
int insert(skip_list l, char *key, value v, int replace_dups);

/* return the value of the key in the first element of the list */
void *first_key(skip_list l);

/* deletes an item from a list... returns 1 if the item was in the list,
   0 if nothing was found */
int delete(skip_list l, void *key);

/* find a key in a list */
int find(skip_list l, void *key, value *v);

/* find the next matching key */
int find_next(skip_list l, void *key, value *v);

/* modify an element... especially fast if we just used find */
void modify(skip_list l, void *key, value v);

/* store the contents of a skip list in a form we can read quickly.
   if the size and copier routines are defined, then this will
   preserve pointer sharing and all kinds of neat things */
void write_skip_list(FILE *f, skip_list l);

/* read a skip list back in from a file */
skip_list read_skip_list(FILE *f,
			 int (*key_compare)(),
			 int (*key_copy)(), int (*key_size)(),
			 int (*data_copy)(), int (*data_size)());

