/******************************************************************
 * display.c -- This file contains all file manipulation routines *
 *              for CS273's Sim6811_GUI.                          *
 *                                                                *
 *                  written by Rob Sinclair                       *
 *                   rsinclai@cs.nmsu.edu                         *
 ******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/font.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gfm.h>
#include "sim6811_ui.h"
#include "defs.h"


extern  sim6811_sim6811Window_objects	    *Sim6811_sim6811Window;
extern  sim6811_winSourceCode_objects	    *Sim6811_winSourceCode;
extern  sim6811_winCrossRef_objects	    *Sim6811_winCrossRef;
extern  sim6811_winCircuitBoard_objects	    *Sim6811_winCircuitBoard;
extern  sim6811_winMemory_objects	    *Sim6811_winMemory;
extern  sim6811_winRegisters_objects	    *Sim6811_winRegisters;
extern  sim6811_winStack_objects	    *Sim6811_winStack;
extern  sim6811_winSymbolTable_objects	    *Sim6811_winSymbolTable;
extern  sim6811_winOutput_objects	    *Sim6811_winOutput;
extern  sim6811_winEditMemory_objects	    *Sim6811_winEditMemory;
extern  sim6811_winMultiStep_objects	    *Sim6811_winMultiStep;
extern  sim6811_winBreakpoints_objects	    *Sim6811_winBreakpoints;
extern  sim6811_winWarning_objects	    *Sim6811_winWarning;
extern  sim6811_winBaseConversion_objects   *Sim6811_winBaseConversion;
extern  sim6811_winPrintUtility_objects	    *Sim6811_winPrintUtility;

/**************************************************************
 * This function activates the File,Assemble menu option that *
 * is used to assemble a program.                             *
 **************************************************************/
void EnableAssembly()
{
    Menu menu;
    int  item;

    menu = (Menu) xv_get(Sim6811_sim6811Window->btnFileMenu,
			 PANEL_ITEM_MENU, NULL);

    item = menu_get(menu, MENU_NTH_ITEM, 8);
    menu_set(item, MENU_INACTIVE, FALSE, NULL);
}

/**************************************************************
 * This function deactivates the File,Assemble menu option    *
 * that is used to assemble a program.                        *
 **************************************************************/
void DisableAssembly()
{
    Menu menu;
    int  item;

    menu = (Menu) xv_get(Sim6811_sim6811Window->btnFileMenu,
			 PANEL_ITEM_MENU, NULL);

    item = menu_get(menu, MENU_NTH_ITEM, 8);
    menu_set(item, MENU_INACTIVE, TRUE, NULL);
}

/**************************************************************
 * This function activates the Edit and Run menu options      *
 * (They are used to run a program)                           *
 **************************************************************/
void EnableRuntime()
{
    /* Activate the Edit Menu, Run Menu, and the File,Assemle
       option. */
    xv_set(Sim6811_sim6811Window->btnEditMenu,
	   PANEL_INACTIVE, FALSE, NULL);
    xv_set(Sim6811_sim6811Window->btnRunMenu,
	   PANEL_INACTIVE, FALSE, NULL);

    /* Disable the Symbol Table, Cross Reference Table, and
       Output Listing options on the Print Utility window. */
/*
    xv_set(Sim6811_winPrintUtility->btn
	   PANEL_INACTIVE, FALSE, NULL);
*/
}

/**************************************************************
 * This function deactivates the Edit and Run menus, and hides*
 * the Multi-Step, Memory Editor, and Breakpoints windows.    *
 * (They all have the potential to affect the runtime system) *
 **************************************************************/
void DisableRuntime()
{
    /* Deactivate the Edit Menu, Run Menu, and the File, Assemble
       option. */
    xv_set(Sim6811_sim6811Window->btnEditMenu,
	   PANEL_INACTIVE, TRUE, NULL);
    xv_set(Sim6811_sim6811Window->btnRunMenu,
	   PANEL_INACTIVE, TRUE, NULL);

    /* Hide all windows capable of affecting the runtime system. */
    xv_set(Sim6811_winBreakpoints->winBreakpoints, 
	   FRAME_CMD_PUSHPIN_IN, FALSE,
	   XV_SHOW, FALSE, NULL);

    xv_set(Sim6811_winMultiStep->winMultiStep, 
	   FRAME_CMD_PUSHPIN_IN, FALSE,
	   XV_SHOW, FALSE, NULL);

    xv_set(Sim6811_winEditMemory->winEditMemory, 
	   FRAME_CMD_PUSHPIN_IN, FALSE,
	   XV_SHOW, FALSE, NULL);


    DisableAssembly();    
}

/**************************************************************
 * This function resets the dynamic info windows (Memory,     *
 * Registers, Stack, and Breakpoints).                        *
 **************************************************************/
void assembly_reset()
{
    int  i, lenDefMem=0, numItems=0;
    char defMem[]="aa aa aa aa aa aa aa aa aa aa aa aa aa aa aa aa  ................";

    lenDefMem = strlen(defMem);

DEBUG(fprintf(stderr, "Beginning the assembly_reset() routine.\n"));

    /* Display the default values in each field */
    xv_set(Sim6811_sim6811Window->sim6811Window, 
	   FRAME_RIGHT_FOOTER, "Cycles:    0", NULL);

    xv_set(Sim6811_winRegisters->txtRegPC, PANEL_VALUE, "0000", NULL);
    xv_set(Sim6811_winRegisters->txtRegSP, PANEL_VALUE, "0000", NULL);
    xv_set(Sim6811_winRegisters->txtRegCCodes, 
	   PANEL_VALUE, "00 (.... ....)", NULL);

    xv_set(Sim6811_winRegisters->txtRegA, PANEL_VALUE, "aa", NULL);
    xv_set(Sim6811_winRegisters->txtRegB, PANEL_VALUE, "aa", NULL);
    xv_set(Sim6811_winRegisters->txtRegD, PANEL_VALUE, "aaaa", NULL);
    xv_set(Sim6811_winRegisters->txtRegX, PANEL_VALUE, "aaaa", NULL);
    xv_set(Sim6811_winRegisters->txtRegY, PANEL_VALUE, "aaaa", NULL);

    xv_set(Sim6811_winMemory->txtMem0, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem1, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem2, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem3, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem4, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem5, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem6, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem7, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem8, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem9, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem10, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem11, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem12, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem13, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem14, PANEL_VALUE, defMem, NULL);
    xv_set(Sim6811_winMemory->txtMem15, PANEL_VALUE, defMem, NULL);


    /* Remove all breakpoints from the Breakpoints window. */
    numItems = (int)xv_get(Sim6811_winBreakpoints->listBreakPts, 
			   PANEL_LIST_NROWS, NULL);

    for (i=0; i<numItems; i++) {
	xv_set(Sim6811_winBreakpoints->listBreakPts, 
	       PANEL_LIST_DELETE, i, NULL);
    }

    /* Disable the Update and Delete buttons on the Breakpoints window */
    xv_set(Sim6811_winBreakpoints->btnBreakPtsUpdate, 
	   PANEL_INACTIVE, TRUE, NULL); 
    xv_set(Sim6811_winBreakpoints->btnBreakPtsDelete, 
	   PANEL_INACTIVE, TRUE, NULL); 
    
DEBUG(fprintf(stderr, "Completed the assembly_reset() routine.\n"));
}


/**************************************************************
 * This function restores all windows to their default state. *
 **************************************************************/
void ResetWindows()
{
    /* Clear the contents of the text windows */
    textsw_reset(Sim6811_winCrossRef->textCrossRef, 0, 0);
    textsw_reset(Sim6811_winSymbolTable->textSymbolTable, 0, 0);
    textsw_reset(Sim6811_winOutput->textOutput, 0, 0);

    /* Clear the source code window twice to unload it */
    textsw_reset(Sim6811_winSourceCode->textSourceCode, 0, 0);
    textsw_reset(Sim6811_winSourceCode->textSourceCode, 0, 0);
    
    assembly_reset();

    DisableRuntime();
}


/************************************************************
 * This function displays the contents of File in the Cross *
 * References Window.                                       *
 ************************************************************/
void displayCrossRefs(char* File)
{
    FILE*        fp;
    int          statResult;
    struct stat  fileInfo;
    char         line[255];

    /* Attempt to open File. If it succeeds, continue */
    if ((fp = fopen(File, "r")) != NULL) {

	/* Get the statistics on File */
	statResult = stat(File, &fileInfo);
	
	/* If the file is not empty, load it */
	if ((statResult == 0) && (fileInfo.st_size != 0)) {

	    /* Skip the first byte (clear screen character) */
	    fgetc( fp );

	    /* Display the rest of the file in the CrossRefs window */
	    while ( fgets(line, 255, fp) ) {
		textsw_insert(Sim6811_winCrossRef->textCrossRef,
			      line, strlen(line));
	    }

	    /* Display the beginning of the new Cross Refs at top of window */
	    window_set(Sim6811_winCrossRef->textCrossRef, TEXTSW_FIRST, 0, 0);
	}
	fclose(fp);
    }
}


/*************************************************************
 * This function displays the contents of File in the Symbol *
 * Table Window.                                             *
 *************************************************************/
void displaySymbolTable(char* File)
{
    FILE*        fp;
    int          statResult;
    struct stat  fileInfo;
    char         line[255];

    /* Attempt to open File. If it succeeds, continue */
    if ((fp = fopen(File, "r")) != NULL) {

	/* Get the statistics on File */
	statResult = stat(File, &fileInfo);
	
	/* If the file is not empty, load it */
	if ((statResult == 0) && (fileInfo.st_size != 0)) {
	    /* Skip the first byte (a clear screen character) */
	    fgetc( fp );

	    /* Display the rest of the file in the Symbol Table window */
	    while ( fgets(line, 255, fp) ) {
		textsw_insert(Sim6811_winSymbolTable->textSymbolTable,
			      line, strlen(line));
	    }

	    /* Display the beginning of the Symbol Table at top of window */
	    window_set(Sim6811_winSymbolTable->textSymbolTable, 
		       TEXTSW_FIRST, 0, 0);
	}
	fclose(fp);
    }
}


/*************************************************************
 * This function displays the contents of File in the Output *
 * Window.                                                   *
 *************************************************************/
void displayOutput(char* File)
{
    FILE*        fp;
    int          statResult, index=0;
    char         line[255];
    struct stat  fileInfo;

    /* Flush the output window's buffer to avoid buffer overload */
    textsw_reset(Sim6811_winOutput->textOutput, 0, 0);

    /* Attempt to open File. If it succeeds, continue */
    if ((fp = fopen(File, "r")) != NULL) {

	/* Get the statistics on File */
	statResult = stat(File, &fileInfo);

	/* Get char index to be used as the new "top of window". */
	index = xv_get(Sim6811_winOutput->textOutput, 
		       TEXTSW_INSERTION_POINT, NULL) + 1;

	/* If the file is empty, assume assembly completed
	   successfully and display "Finished" in Output window.
	   Otherwise, display the assembly results in Output */
/*	if ((statResult == 0) && (fileInfo.st_size == 0)) {
	    textsw_insert(Sim6811_winOutput->textOutput, 
			  " Finished\n", strlen(" Finished\n"));
*/
	    /* Increment the char index by 9 to get past " Finished\n". */
	    index += 9;
/*	}
	else {
*/	    /* Insert the contents of the file in the Output window */
	    textsw_insert(Sim6811_winOutput->textOutput, "\n", strlen("\n"));
	    while ( fgets(line, 255, fp) ) {
		textsw_insert(Sim6811_winOutput->textOutput,
			      line, strlen(line));
	    }
/*	}
*/
	/* Display the beginning of the new output at top of window */
	xv_set(Sim6811_winOutput->textOutput, TEXTSW_FIRST, index, NULL);

	fclose(fp);
    }
}


/****************************************************************
 * This function changes the position of each window and sets   *
 * other custom settings for improved display (such as the      *
 * field length of each memory cell).                           *
 ****************************************************************/
void CustomizeWindows()
{
    int     maxMemFieldWidth = 68, item;
    Menu    menu;
    Xv_Font font;

    /* Create a fixed-width font for the register and memory windows. */
    font = (Xv_Font)xv_find(NULL, FONT, 
			    FONT_FAMILY, FONT_FAMILY_LUCIDA_FIXEDWIDTH,
			    FONT_STYLE,  FONT_STYLE_NORMAL, NULL);

    /* Set the position of each window */
    xv_set(Sim6811_sim6811Window->sim6811Window, XV_X, 415, XV_Y, 50, NULL);
    xv_set(Sim6811_winSymbolTable->winSymbolTable, XV_X, 30, XV_Y, 50, NULL);
    xv_set(Sim6811_winCrossRef->winCrossRef, XV_X, 30, XV_Y, 531, NULL);
    xv_set(Sim6811_winOutput->winOutput, XV_X, 331, XV_Y, 531, NULL);
    xv_set(Sim6811_winStack->winStack, XV_X, 965, XV_Y, 531, NULL);
    xv_set(Sim6811_winMemory->winMemory, XV_X, 550, XV_Y, 205, NULL);
    xv_set(Sim6811_winRegisters->winRegisters, XV_X, 880, XV_Y, 50, NULL);
    xv_set(Sim6811_winCircuitBoard->winCircuitBoard, XV_X,345, XV_Y,140, NULL);
    xv_set(Sim6811_winWarning->winWarning, XV_X, 430, XV_Y, 260, NULL);
    xv_set(Sim6811_winBreakpoints->winBreakpoints, XV_X, 470, XV_Y, 210, NULL);
    xv_set(Sim6811_winEditMemory->winEditMemory, XV_X, 510, XV_Y, 140, NULL);
    xv_set(Sim6811_winMultiStep->winMultiStep, XV_X, 460, XV_Y, 160, NULL);
    xv_set(Sim6811_winSourceCode->winSourceCode, XV_X, 30, XV_Y, 205, NULL);
    xv_set(Sim6811_winBaseConversion->winBaseConversion, 
	   XV_X, 385, XV_Y, 135, NULL);
    xv_set(Sim6811_winPrintUtility->winPrintUtility, XV_X,345, XV_Y,140, NULL);

    /* Remove the 'FILE' option from the Source Code window's popup menu. */
    menu = (Menu) xv_get(Sim6811_winSourceCode->textSourceCode,
			 TEXTSW_MENU, NULL);
    xv_set(menu, MENU_REMOVE, 2, NULL);

    /* Remove the 'FILE' option from the other windows' popup menu. */
    menu = (Menu) xv_get(Sim6811_winOutput->textOutput,
			 TEXTSW_MENU, NULL);
    xv_set(menu, MENU_REMOVE, 2, NULL);


    xv_set(Sim6811_winPrintUtility->listPrinterQueue, XV_FONT, font, NULL); 


    /* Initially inactivate all assembly and run options. */
    xv_set(Sim6811_winBreakpoints->btnBreakPtsDelete,
	   PANEL_INACTIVE, TRUE, NULL);
    xv_set(Sim6811_winBreakpoints->btnBreakPtsUpdate,
	   PANEL_INACTIVE, TRUE, NULL);
    xv_set(Sim6811_winBreakpoints->txtBreakPtTimes, 
	   PANEL_INACTIVE, TRUE, NULL);

    /* Set the custom settings for the Register window */
    xv_set(Sim6811_winRegisters->txtRegCCodes, XV_FONT, font, NULL); 

    /* Set the custom settings for the Memory window */
    xv_set(Sim6811_winMemory->txtMem0, 
	   PANEL_VALUE_UNDERLINED, FALSE,
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem1, 
	   PANEL_VALUE_UNDERLINED, FALSE,
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem2, 
	   PANEL_VALUE_UNDERLINED, FALSE, 
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem3, 
	   PANEL_VALUE_UNDERLINED, FALSE, 
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem4, 
	   PANEL_VALUE_UNDERLINED, FALSE, 
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem5, 
	   PANEL_VALUE_UNDERLINED, FALSE, 
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem6, 
	   PANEL_VALUE_UNDERLINED, FALSE, 
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem7, 
	   PANEL_VALUE_UNDERLINED, FALSE, 
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem8, 
	   PANEL_VALUE_UNDERLINED, FALSE, 
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem9, 
	   PANEL_VALUE_UNDERLINED, FALSE, 
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem10, 
	   PANEL_VALUE_UNDERLINED, FALSE,
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem11, 
	   PANEL_VALUE_UNDERLINED, FALSE,
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem12, 
	   PANEL_VALUE_UNDERLINED, FALSE,
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem13, 
	   PANEL_VALUE_UNDERLINED, FALSE,
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem14, 
	   PANEL_VALUE_UNDERLINED, FALSE,
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
    xv_set(Sim6811_winMemory->txtMem15, 
	   PANEL_VALUE_UNDERLINED, FALSE,
	   PANEL_VALUE_DISPLAY_LENGTH, maxMemFieldWidth, 
	   PANEL_VALUE_STORED_LENGTH, maxMemFieldWidth, 
	   XV_FONT, font, NULL); 
}    

