/***************************************************************
 * file.c -- This file contains all file manipulation routines *
 *           for CS273's Sim6811_GUI.                          *
 *                                                             *
 *                 written by Rob Sinclair                     *
 *                  rsinclai@cs.nmsu.edu                       *
 ***************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gfm.h>
#include "sim6811_ui.h"
#include "defs.h"


extern  sim6811_sim6811Window_objects	*Sim6811_sim6811Window;
extern  sim6811_winSourceCode_objects	*Sim6811_winSourceCode;
extern  sim6811_winCrossRef_objects	*Sim6811_winCrossRef;
extern  sim6811_winCircuitBoard_objects	*Sim6811_winCircuitBoard;
extern  sim6811_winMemory_objects	*Sim6811_winMemory;
extern  sim6811_winRegisters_objects	*Sim6811_winRegisters;
extern  sim6811_winStack_objects	*Sim6811_winStack;
extern  sim6811_winSymbolTable_objects	*Sim6811_winSymbolTable;
extern  sim6811_winOutput_objects	*Sim6811_winOutput;
extern  sim6811_winEditMemory_objects	*Sim6811_winEditMemory;
extern  sim6811_winWarning_objects	*Sim6811_winWarning;

extern char    currFile[];
extern char    currPath[];
extern char    asmFile[];
extern char    login[];
extern int     EditsPending;

extern void    (*discard_function)();

/*************************************************************
 * This function updates the global variable currFile and    *
 * displays its name in the footer of the Source Code window *
 * and the main application window.                          *
 *************************************************************/
void SetCurrFile(char *File)
{
    char  *ptr, *pathname;
    char  temp[MAXPATHLEN]="~", tokens[] = "/";

    /* Update the global variable */
    strcpy(currFile, File);

    /* Update footer of Source Code window with fully-
       qualified filename (beginning with user's home dir) */
    if ((pathname = strstr(File, login)) == NULL) {
	xv_set(Sim6811_winSourceCode->winSourceCode, 
	       FRAME_LEFT_FOOTER, currFile, 0);
    }
    else {
	strcat(temp, pathname);
	xv_set(Sim6811_winSourceCode->winSourceCode, 
	       FRAME_LEFT_FOOTER, temp, 0);
    }

    /* Extract the filename from currFile (includes path)
       and update the Main window's footer. */
    ptr = strtok(File, tokens);
    if (strcmp("Untitled", File) == 0) {
	xv_set(Sim6811_sim6811Window->sim6811Window,
	       FRAME_LEFT_FOOTER, currFile, 0);
    }
    else {
	while ((ptr = strtok(NULL, tokens)) != NULL) 
	  strcpy(asmFile, ptr);
 
	xv_set(Sim6811_sim6811Window->sim6811Window,
	       FRAME_LEFT_FOOTER, asmFile, 0);
    }
}


/**************************************************************
 * This function closes the current file and opens a new file *
 **************************************************************/
void fileClose()
{
    ResetWindows();

    /* Reset the currently selected filename to 'Untitled'. */
    SetCurrFile("Untitled");
    DisableRuntime();
    EditsPending = FALSE;
}


/*************************************************************
 * This function checks filename's extension.  If it is .asm *
 * the File,Assemble menu option is enabled.  Otherwise this *
 * menu option is disabled.                                  *
 *************************************************************/
void VerifyAssembly(char *filename)
{
    if ( ASM_FILE(filename) )
       EnableAssembly();
    else
       DisableAssembly();
}


/************************************************************
 * This function is called to open a file and update the    *
 * state of the simulator.                                  *
 ************************************************************/
void openFile(char *File) 
{
    ResetWindows();
    xv_set(Sim6811_winSourceCode->textSourceCode, 
	   TEXTSW_FILE, File, XV_SHOW, TRUE, NULL);

    /* Update the global variable currFile */
    SetCurrFile(File);

    /* If the file's extension is not .asm, make the 
       File, Assemble menu item INACTIVE. Otherwise,
       make it ACTIVE. */
    VerifyAssembly( currFile );

    EditsPending = FALSE;
}


/************************************************************
 * Callback function for the FileOpen file selector object. *
 ************************************************************/
int gfm_OpenProc(ip, dir, file)
     gfm_popup_objects  *ip;
     char               *dir;
     char               *file;
{
    char selFile[MAXPATHLEN];

    /* Construct the fully-qualified file name */
    sprintf(selFile, "%s/%s", dir, file);

    /* Actually open the file */
    openFile(selFile);

    return GFM_OK;
}
/************************************************************
 * Callback function for the FileSave file selector object. *
 ************************************************************/
int gfm_SaveProc(ip, dir, file)
     gfm_popup_objects  *ip;
     char               *dir;
     char               *file;
{
    char selFile[MAXPATHLEN];

    /* Construct the fully-qualified file name */
    sprintf(selFile, "%s/%s", dir, file);
    textsw_store_file(Sim6811_winSourceCode->textSourceCode, 
		      selFile, 100, 100);

    /* Update the global variable currFile */
    SetCurrFile(selFile);

    /* If the file's extension is .asm, make the 
       File, Assemble menu item ACTIVE. */
    VerifyAssembly( currFile );

    EditsPending = FALSE;

    return GFM_OK;
}


/************************************************************
 * This function attempts to assemble the currently active  *
 * file and display the results in the Output, Symbol Table *
 * and Cross Reference windows.                             *
 ************************************************************/
int assemble(char* program, int listing)
{
    char         Command[MAXPATHLEN], temp[MAXPATHLEN], 
                 tempOutput[MAXPATHLEN] = "/tmp/sim6811OutXXXXXX", 
                 tempSymbol[MAXPATHLEN] = "/tmp/sim6811SymXXXXXX",
                 tempCrossRef[MAXPATHLEN] = "/tmp/sim6811CrRXXXXXX",
                 s19file[MAXPATHLEN];
    int          len, statOutput, statS19, assembled=0, tempLen=0,
                 status = 13;
    struct stat  outputFileInfo, s19FileInfo;
    char         path[MAXPATHLEN], filename[MAXPATHLEN], *ptr;

    /* Clear the Output, CrossRef and SymbolTable windows and 
       then tell user we're trying to assemble */
    textsw_reset(Sim6811_winCrossRef->textCrossRef, 0, 0);
    textsw_reset(Sim6811_winSymbolTable->textSymbolTable, 0, 0);

    textsw_reset(Sim6811_winOutput->textOutput, 0, 0);
    textsw_insert(Sim6811_winOutput->textOutput, 
		  "\nAssembling ...", strlen("\nAssembling ..."));

    len = strlen(program);

    /* Extract the path and filename of .asm file from "program" */
    strcpy(path, program);
    path[(strlen(path) - strlen(asmFile))] = '\0';
    strcpy(currPath, path);
    
    strcpy(filename, strstr(program, asmFile));
    filename[strlen(filename) - 4] = '\0';

    DEBUG(fprintf(stderr, "\nProgram: %s\nPath: %s\nFilename: %s\n",
		  program, path, filename));

    /* If the file is untitled, don't assemble it.  Otherwise
       attempt to assemble it. */
    if (strcmp("Untitled", program) == 0) {
	textsw_insert(Sim6811_winOutput->textOutput, 
"\nCannot Assemble:  Either save the current file or load an assembly file.\n",
 strlen(
   "\nCannot Assemble:  Either save the current file or load an assembly file.\n"));
	return 0;
    }
    else
      if ( ASM_FILE(program) ) {
	  /* The as11 assembler is apparently unable to accept 
	     fully-qualified paths to .asm files.  Therefore, we
	     must first change to the directory containing the 
	     current .asm file, and then assemble the file. */

	  /* Construct the assemble command and execute it. */
	  mktemp(tempOutput);
	  if (tempOutput == NULL) {
	      textsw_insert(Sim6811_winOutput->textOutput, 
		  "\nCannot create temp file needed for assembly.\n", 
		  strlen("\nCannot create temp file needed for assembly.\n"));
	      return 0;
	  }
	  else {
	      if (listing) {
		  sprintf(Command, "cd %s ; as11 %s -l > %s", 
			  path, asmFile, tempOutput);
	      }
	      else {
		  sprintf(Command, "cd %s ; as11 %s > %s", 
			  path, asmFile, tempOutput);
	      }

	      DEBUG(fprintf(stderr, "\nAbout to execute:\n%s\n", Command));
	      status = system(Command);
	  }

	  /* Motorola's assembler does not use the entire .asm filename
	     for the .s19 filename.  It only uses the characters that
	     preceed the first period.  So, assembling 'test1.test.asm'
	     produces a .s19 file called 'test1.s19'. */
	  strcpy(temp, filename);
	  if ((ptr = strtok(temp, ".")) == NULL)
	    sprintf(s19file, "%s%s.s19", path, filename);
	  else
	    sprintf(s19file, "%s%s.s19", path, ptr);

	  DEBUG(fprintf(stderr,"About to test .s19 file: %s\n", s19file));

	  /* If the assemble command returned status 0, conclude
	     conclude that assembly succeeded and continue with Symbol 
	     Table and Cross Reference Table. */
	  if ( status == 0 ) {

	      assembled = 1;

	      /* Build the symbol table assembler command and execute it. */
	      mktemp(tempSymbol);
	      if (tempSymbol == NULL) {
		  textsw_insert(Sim6811_winOutput->textOutput, 
		     "\nCannot create temp file for Symbol Table.\n", 
		     strlen("\nCannot create temp file for Symbol Table.\n"));

		  assembled = 0;
	      }
	      else {
		  sprintf(Command, "cd %s ; as11 %s -s -nol > %s", 
			  path, asmFile, tempSymbol);

		  DEBUG(fprintf(stderr, "\nAbout to execute:\n%s\n", Command));
		  system(Command);

		  /* Load the Symbol Table. */
		  displaySymbolTable(tempSymbol);

		  /* Delete the temp file created above */
		  sprintf( Command, "rm %s", tempSymbol );
		  system( Command );
	      }

	      /* Build the crossref assembler command and execute it. */
	      mktemp(tempCrossRef);
	      if (tempCrossRef == NULL) {
		 textsw_insert(Sim6811_winOutput->textOutput, 
		   "\nCannot create temp file for Cross References Table.\n", 
		   strlen("\nCannot create temp file for Cross References Table.\n"));

		 assembled = 0;
	      }
	      else {
		  sprintf(Command, "cd %s ; as11 %s -cre -nol > %s", 
			  path, asmFile, tempCrossRef);

		  DEBUG(fprintf(stderr, "\nAbout to execute:\n%s\n", Command));
		  system(Command);

		  /* Load the Cross Reference Table. */
		  displayCrossRefs(tempCrossRef);

		  /* Delete the temp file created above */
		  sprintf( Command, "rm %s", tempCrossRef );
		  system( Command );
	      }

	      /* Notify user that assembly succeeded. */
	      textsw_insert(Sim6811_winOutput->textOutput, 
			    " Finished\n", strlen(" Finished\n"));

	      /* If the Assembly With Listing option was chosen,
		 display the assembler's output in the Output window. */
	      if ( listing ) {
		  /* Display assembler output in the Output window */
		  displayOutput(tempOutput);
	      }
	  }
	  else {
	      /* Notify user that assembly failed. */
	      textsw_insert(Sim6811_winOutput->textOutput, 
			    " Failed\n", strlen(" Failed\n"));

	      DEBUG(sprintf(Command,"cp %s reasonForFailure.txt",tempOutput));
	      DEBUG(system(Command));

	      /* Display assembler output in the Output window */
	      displayOutput(tempOutput);

	      assembled = 0;
	  }

	  /* Remove the assembly temp file created above. */
	  sprintf(Command, "rm %s", tempOutput);
	  system( Command );
      }
      else {
	  /* Display error message */
	  textsw_insert(Sim6811_winOutput->textOutput, 
    "\nCannot Assemble:  Load an assembly file before selecting Assemble.\n", 
    strlen(
    "\nCannot Assemble:  Load an assembly file before selecting Assemble.\n"));
      }

    /* Return success if assembly completed. */
    return assembled;
}

