/***************************************************************
 * print.c -- This file contains all routines used for the     *
 *            Print Utility window in CS273's Sim6811_GUI.     *
 *                                                             *
 *                 written by Rob Sinclair                     *
 *                  rsinclai@cs.nmsu.edu                       *
 ***************************************************************/

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <gfm.h>
#include "sim6811_ui.h"
#include "defs.h"


extern  sim6811_winCrossRef_objects	*Sim6811_winCrossRef;
extern  sim6811_winSymbolTable_objects	*Sim6811_winSymbolTable;
extern  sim6811_sim6811Window_objects	*Sim6811_sim6811Window;
extern  sim6811_winOutput_objects	*Sim6811_winOutput;
extern  sim6811_winWarning_objects	*Sim6811_winWarning;
extern  sim6811_winPrintUtility_objects	*Sim6811_winPrintUtility;

extern  char  currFile[];
extern  char  currPath[];
extern  char  asmFile[];
extern  char *sys_errlist[];
extern  int   sys_nerr;

char   printers[50][10];
int    numPrinters=1;

/************************************************************
 * This function initializes the Print Utility window.
 ************************************************************/
int InitPrint()
{
    int   i=0;
    char  *ptr, text[256], junk[255];
    char  command[255], tokens[]="|", temp[255]="";
    FILE  *printcap;

    /* Parse the /etc/printcap file to get the list of 
       available printers. */
    if ( (printcap = fopen("/etc/printcap", "r")) == NULL) {
	/* Only allow printer access to line printer */
	xv_set(Sim6811_winPrintUtility->setPrinter,
	       PANEL_CHOICE_STRING, 0, "Default Printer", NULL);

	strcpy(printers[0], "lp");
	numPrinters = 1;
    }
    else {
	i=0;
	while ((fgets(text, 255, printcap) != NULL) && i<50) {

	    if (!strncmp(text, "chip|HP Color Postscript Printer:", 33)) {
		/* Don't insert this printer!  It's ID is bad! */
	    }
	    else
	    if (sscanf(text, "%[^|]|%[^|]|%[^|]|%[^:]:\\", 
		       printers[i], junk, junk, temp) == 4) {
DEBUG(fprintf(stderr, "Succeeded in Case #1.\n"));
DEBUG(fprintf(stderr, "Adding Printer: %s, (%s)\n", temp, printers[i]));
		xv_set(Sim6811_winPrintUtility->setPrinter,
		       PANEL_CHOICE_STRING, i, temp, NULL);

		/* If the current printer is 'lp' select it. */
		if (strcmp(printers[i], "lp") == 0)
		  xv_set(Sim6811_winPrintUtility->setPrinter,
			 PANEL_VALUE, i, PANEL_DEFAULT_VALUE, i, NULL);
		i++;
	    }
	    else
	    if (sscanf(text, "%[^|]|%[^|]|%[^:]:\\", 
		       printers[i], junk, temp) == 3) {
DEBUG(fprintf(stderr, "Succeeded in Case #2.\n"));
DEBUG(fprintf(stderr, "Adding Printer: %s, (%s)\n", temp, printers[i]));
		xv_set(Sim6811_winPrintUtility->setPrinter,
		       PANEL_CHOICE_STRING, i, temp, NULL);

		/* If the current printer is 'lp' select it. */
		if (strcmp(printers[i], "lp") == 0)
		  xv_set(Sim6811_winPrintUtility->setPrinter,
			 PANEL_VALUE, i, PANEL_DEFAULT_VALUE, i, NULL);
		i++;
	    }
	    else
	    if (sscanf(text, "%[^|]|%[^:]:\\", printers[i], temp) == 2) {
DEBUG(fprintf(stderr, "Succeeded in Case #3.\n"));
DEBUG(fprintf(stderr, "Adding Printer: %s, (%s)\n", temp, printers[i]));

		xv_set(Sim6811_winPrintUtility->setPrinter,
		       PANEL_CHOICE_STRING, i, temp, NULL);

		/* If the current printer is 'lp' select it. */
		if (strcmp(printers[i], "lp") == 0)
		  xv_set(Sim6811_winPrintUtility->setPrinter,
			 PANEL_VALUE, i, PANEL_DEFAULT_VALUE, i, NULL);
		i++;
	    }
	}
    }

    fclose(printcap);
}


/************************************************************
 * This function copies everthing from sourceFile into the
 * file pointed to by destFile.  The primary reason for this
 * function is to skip the first character (a control char)
 * in the Symbol and Cross Reference tables.
 ************************************************************/
void dumpContents(char *sourceFile, FILE *destFile) 
{
    FILE*        fp;
    int          statResult;
    struct stat  fileInfo;
    char         line[255];

    /* Attempt to open sourceFile. If it succeeds, continue */
    if ((fp = fopen(sourceFile, "r")) != NULL) {

	/* Get the statistics on sourceFile */
	statResult = stat(sourceFile, &fileInfo);
	
	/* If the file is not empty, load it */
	if ((statResult == 0) && (fileInfo.st_size != 0)) {

	    /* Skip the first byte (clear screen character) */
	    fgetc( fp );

	    /* Display the rest of the file in the CrossRefs window */
	    while ( fgets(line, 255, fp) ) {
		fprintf(destFile, "%s", line);
	    }
	}
	fclose(fp);
    }
}


/************************************************************
 * This function takes two filenames as arguments.  The file
 * called 'source' contains the information that should be
 * printed.  'destFile' is the name of the file that should
 * contain the contents of 'source' immediately after the
 * header information (name of file being printed, and the
 * user's username).  However, if the header information
 * cannot be inserted, this file will not be used. 
 * The char* returned contains the filename of the file that 
 * should be printed.
 ************************************************************/
char *prepDocument(char *title, char *source, 
		   char *tempFile, char *destFile)
{
    char  command[MAXPATHLEN];
    FILE  *tmpfp;

    int xpos;

    xpos=(int)((double)40.0-(((double)strlen(title)+8.0)/2.0))+strlen(title);
    if (xpos < 0) xpos = 0;

    if ((tmpfp = fopen(tempFile, "w")) == NULL) {
	return source;
    }
    else {
	fprintf(tmpfp, "\n%*s [%s]\n\n", xpos, title, getenv("LOGNAME"));

	/* If file is the source code or output listing, cat them
	   with the file containing the header info. */
	if (strstr(title, "Source Code for") || 
	    strstr(title, "Output Listing for")) {
	    /* Close the temp file */
	    fclose(tmpfp);
	    sprintf(command, "cat %s %s > %s", 
		    tempFile, source, destFile);

	    if (system(command)) {
		/* Insertion of header failed, so just use source file. */
		return source;
	    }
	    else {
		/* Print file with header */
		return destFile;
	    }
	}
	else {
	    /* The file must be either the Symbol Table or the
	       Cross Reference Table.  So, directly append the
	       information from the source file to the header. */
	    dumpContents(source, tmpfp);

	    /* Close the temp file */
	    fclose(tmpfp);
	    return tempFile;
	}
    }
    return source;
}


/************************************************************
 * This function prints the information specified in the
 * Print Utility window.  It allows the user to specify
 * the printer, number of copies, and which of the following
 * documents should be printed:
 *    - Source Code
 *    - Output Listing
 *    - Cross Reference Table
 *    - Symbol Table
 ************************************************************/
int DoPrint()
{
    int   printerID, copies, value, status;
    char  printer[10], command[MAXPATHLEN], file[MAXPATHLEN],
          error[512], title[80],
          tempFile[MAXPATHLEN] = "/tmp/sim6811FileXXXXXX",
          tempDump[MAXPATHLEN] = "/tmp/sim6811XXXXXX",
          tempHeader[MAXPATHLEN] = "/tmp/sim6811HeaderXXXXXX";
    FILE  *tmpfp;

    /* Create a temp file to use for printing. */
    mktemp(tempHeader);
    mktemp(tempDump);
    mktemp(tempFile);
    if (tempFile == NULL || tempHeader == NULL || tempDump == NULL) {
	textsw_insert(Sim6811_winOutput->textOutput, 
		      "Error: Cannot create printer temp files.\n", 
		      strlen("Error: Cannot create printer temp files.\n"));

	return 0;
    }

    /* Get the name for the selected printer */
    printerID = (int)xv_get(Sim6811_winPrintUtility->setPrinter, 
			    PANEL_VALUE, NULL);

    /* Get the number of copies (between 1 and 10) */
    copies = (int)xv_get(Sim6811_winPrintUtility->textNumCopies, 
			 PANEL_VALUE, NULL);

    /* Print each document that is checked in the Print 
       Utility window */
    value = (int) xv_get(Sim6811_winPrintUtility->setPrintSelection,
			 PANEL_VALUE, 0, NULL);

    /* Is 'Source Code' checked? */
    if (value & 01) { 

	/* Construct the print command. */
	sprintf(title, "Source Code for %s", asmFile);
	sprintf(command, "lpr -P%s -#%d %s", 
		printers[printerID], copies,
		prepDocument(title, currFile, tempHeader, tempFile));

	/* Print source code file (with or without header). */
	if (status = system(command))
	  if (status <= sys_nerr) {
	      sprintf(error, "Print Failed: %s\n", sys_errlist[status]);
	      textsw_insert(Sim6811_winOutput->textOutput, 
			    error, strlen(error));
	  }
    }

    value >>= 1;

    /* Is 'Output Listing' checked? */
    if (value & 01) {
	sprintf(command, "cd %s ; as11 %s -l > %s", 
		currPath, asmFile, tempDump);
	system(command);

	/* Construct the print command. */
	sprintf(title, "Output Listing for %s", asmFile);
	sprintf(command, "lpr -P%s -#%d %s", 
		printers[printerID], copies,
		prepDocument(title, tempDump, tempHeader, tempFile));

	/* Print file (with or without header). */
	if (status = system(command))
	  if (status <= sys_nerr) {
	      sprintf(error, "Print Failed: %s\n", sys_errlist[status]);
	      textsw_insert(Sim6811_winOutput->textOutput, 
			    error, strlen(error));
	  }
    }

    value >>= 1;

    /* 'Cross Reference Table' checked? */
    if (value & 01) {
	sprintf(command, "cd %s ; as11 %s -cre -nol > %s", 
		currPath, asmFile, tempDump);
	system(command);

	/* Construct the print command. */
	sprintf(title, "Cross Reference Table for %s", asmFile);
	sprintf(command, "lpr -P%s -#%d %s", 
		printers[printerID], copies,
		prepDocument(title, tempDump, tempHeader, tempFile));

	/* Print file (with or without header). */
	if (status = system(command))
	  if (status <= sys_nerr) {
	      sprintf(error, "Print Failed: %s\n", sys_errlist[status]);
	      textsw_insert(Sim6811_winOutput->textOutput, 
			    error, strlen(error));
	  }
    }

    value >>= 1;

    /* 'Symbol Table' checked? */
    if (value & 01) {
	sprintf(command, "cd %s ; as11 %s -s -nol > %s", 
		currPath, asmFile, tempDump);
	system(command);

	/* Construct the print command. */
	sprintf(title, "Symbol Table for %s", asmFile);
	sprintf(command, "lpr -P%s -#%d %s", 
		printers[printerID], copies,
		prepDocument(title, tempDump, tempHeader, tempFile));

	/* Print file (with or without header). */
	if (status = system(command))
	  if (status <= sys_nerr) {
	      sprintf(error, "Print Failed: %s\n", sys_errlist[status]);
	      textsw_insert(Sim6811_winOutput->textOutput, 
			    error, strlen(error));
	  }
    }

    /* Delete the printer temp file. */
    sprintf(command, "rm -f %s %s %s", tempFile, tempDump, tempHeader);
    system(command);
    
    /* Return success if print completed. */
    return 1;
}


/************************************************************
 * This function displays the queue information for the 
 * currently selected printer.  It uses the lpq command.
 ************************************************************/
void DisplayStatus()
{
    FILE  *tmpfp;
    int   i, items, printerID, line=0;
    char  command[MAXPATHLEN], text[256], status[500],
          tempInfo[MAXPATHLEN] = "/tmp/sim6811InfoXXXXXX";

    /* Clear the printer status list's contents. */
    xv_set(Sim6811_winPrintUtility->winPrintUtility,
	   FRAME_LEFT_FOOTER, "", NULL);
    items = (int)xv_get(Sim6811_winPrintUtility->listPrinterQueue,
			PANEL_LIST_NROWS, NULL);

    for (i=0; i<=items; i++)
      xv_set(Sim6811_winPrintUtility->listPrinterQueue,
	     PANEL_LIST_DELETE, 0, NULL);

    /* Create a temp file to use for retreiving printer info. */
    mktemp(tempInfo);
    if (tempInfo == NULL) {
	textsw_insert(Sim6811_winOutput->textOutput, 
		      "Error: Cannot create printer temp files.\n", 
		      strlen("Error: Cannot create printer temp files.\n"));
	return;
    }

    /* Get the name for the selected printer */
    printerID = (int)xv_get(Sim6811_winPrintUtility->setPrinter, 
			    PANEL_VALUE, NULL);

    sprintf(command, "lpq -P%s > %s", printers[printerID], tempInfo);
    if (!system(command) && (tmpfp = fopen(tempInfo, "r"))!=NULL) {
	line = 0;
	while (fgets(text, 255, tmpfp) != NULL) {
	    if (line==0) {
		/* Display the printer's status in the footer. */
		xv_set(Sim6811_winPrintUtility->winPrintUtility,
		       FRAME_LEFT_FOOTER, text, NULL);
		++line;
	    }
	    else {
		if (strcmp(text, "Rank   Owner      Job  Files                                 Total Size\n") && !strstr(text, ": Warning: ") &&
		    strcmp(text, "no entries\n") && (strlen(text)>8)) {
		    xv_set(Sim6811_winPrintUtility->listPrinterQueue,
			   PANEL_LIST_INSERT, line-1, 
			   PANEL_LIST_STRING, line-1, text, NULL);
		    ++line;
		}
	    }
	}
    }

    /* Remove the temp files. */
    sprintf(command, "rm -f %s", tempInfo);
    system(command);
}


/************************************************************
 * This function retrieves the information for the job that
 * is currently selected.  Then, it extracts the job # and 
 * checks to make sure the user owns the job.  If so, the
 * job is killed and removed from the queue's status.
 ************************************************************/
void KillJob()
{
    int  selection, job, printerID;
    char rank[10], owner[15], text[80], command[255], 
         printer[10], info[80];

    /* Get the name for the selected printer */
    printerID = (int)xv_get(Sim6811_winPrintUtility->setPrinter, 
			    PANEL_VALUE, NULL);
    strcpy(printer, printers[printerID]);

    /* Find out which item is selected */
    selection = (int)xv_get(Sim6811_winPrintUtility->listPrinterQueue, 
			    PANEL_LIST_FIRST_SELECTED, NULL);

    if (selection != -1) {
	/* Get the Job number from the selected breakpoint */
	strcpy(info,(char*)xv_get(Sim6811_winPrintUtility->listPrinterQueue,
				     PANEL_LIST_STRING, selection, NULL));

	if (!strlen(info) || 
	    (sscanf(info, "%s %s %d %[^\n]\n", rank, owner, &job, text) != 4)){
	    textsw_insert(Sim6811_winOutput->textOutput, 
		  "Error: A valid print job must be selected.\n", 
		  strlen("Error: A valid print job must be selected.\n"));
	    return;
	}

	/* If the user owns the job, kill it. */
	if (!strcmp(owner, getenv("LOGNAME"))) {
	    sprintf(command, "lprm -P%s %d", printer, job);

	    if (!system(command)) {
		xv_set(Sim6811_winPrintUtility->listPrinterQueue,
		       PANEL_LIST_DELETE, selection, NULL);
	    }
	}
	else {
	    textsw_insert(Sim6811_winOutput->textOutput, 
		  "Error: You do not own the selected print job.\n", 
		  strlen("Error: You do not own the selected print job.\n"));
	}
    }
}

